#include <mega128.h>
#include "sub/io.c"    
#include "sub/sys.c"
#include "sub/i1813lcd.c"
#include "sub/ov9640.c"

// Internal RC Oscillator Mode 
//  CKSEL3 CKSEL2 CKSEL1 CKSEL0
//    0      1      0      0     8MHz
        
         
void lcd_clear(void)
{                  
  unsigned int n,m;
  
  LcdSetWindow(0,0,160,128);
  LcdDataStart();
  for(n=0;n<128;n++)
   {  for(m=0;m<160;m++) { LcdData(0xffff); }
   }
  LcdDataEnd();

}


    
void main(void)
{ 
  unsigned int cmd;
  unsigned char cam_data;
  unsigned int n,m;     

  io_init();
  Sini(0,384,0);

  LcdBl(1);
  LcdInit();                
  LcdSetDirection(270);
  lcd_clear();

  EnableAutoSetCM(); 
  SetRGB565QQVGA();

  while(1)
  {  
     put_string0("\n>"); cmd=getch0();
     if(cmd=='1')
      {  SetRGB565QQVGA();
         put_string0("QQVGA-RGB565");
      }
     else if(cmd=='2')
      {  SetRawDataQQVGA();
         put_string0("QQVGA-RawData");
      }
     else if(cmd=='r') 
      {  CmReadyCapture(1);  CmReadyCapture(0);  put_string0(" ReadyCapture"); 
      }
     else if(cmd=='s') 
      {  CmStartCapture(1);  CmStartCapture(0);  put_string0(" Start..."); 
         while(CmEndCapture()==0); put_string0("Completed.");  
      }
     else if(cmd=='v')
      {  put_string0("Display LCD(QQVGA-RGB565)");
         CmReadyCapture(1);  CmReadyCapture(0);  // CLEAR Address...
      
         LcdSetWindow(0,4,160,120);
         LcdDataStart();
         for(n=0;n<120;n++)
           {  
              for(m=0;m<160;m++)
                {
                   CmRdData(1);     
                   #asm("nop")
                   cam_data=CmData();
                   LcdDataH(cam_data);
                   CmRdData(0);
                   
                   CmRdData(1);
                   #asm("nop")
                   cam_data=CmData();
                   LcdDataL(cam_data);
                   CmRdData(0);       
                   
                   LcdWr(1); LcdWr(0);
                 }
           }
         LcdDataEnd();  
       }
     else if(cmd=='b')
       { put_string0("Display LCD(QQVGA-RawData)");
         CmReadyCapture(1);  CmReadyCapture(0);  // CLEAR Address...

         LcdSetWindow(0,4,160,120);
         LcdDataStart();
         for(n=0;n<120;n++)
           {  
              for(m=0;m<160;m++)
                {
                   CmRdData(1);     
                   #asm("nop")
                   cam_data=CmData();
                   CmRdData(0);
                   
    		   if(n%2 == 0)  
		     {  if(m%2 == 0) // G
		         { LcdDataH((cam_data>>5)&0x07); 
		           LcdDataL((cam_data<<6)&0xe0);
		         }
		        else   // B
		         { LcdDataH(0x00);
		           LcdDataL((cam_data>>3)&0x1f);
		         }
		     } 
		   else
		     {  if(m%2 == 0)  // R
		         { LcdDataH(cam_data&0xf8);
		           LcdDataL(0x00);
		         }
		        else   // G
		         { LcdDataH((cam_data>>5)&0x07); 
		           LcdDataL((cam_data<<6)&0xe0);
		         }
		     }
                     LcdWr(1); LcdWr(0);               
                }
           }
         LcdDataEnd();     
       
       }
     else
       {  put_string0("Invalid command.");
          put_string0("\n 1 : Set QQVGA-RGB565 mode");
          put_string0("\n 2 : Set QQVGA-RawData mode");
          put_string0("\n r : Ready Capture");
          put_string0("\n s : Start Capture");
          put_string0("\n v : Display LCD(QQVGA-RGB565)");
          put_string0("\n b : Display LCD(QQVGA-RawData)");
       }
     
  }
}

